@@ -32,6 +32,7 @@ class AccessSatellite(Satellite):
         *args,
         generation_duration: float = 600.0,
         initial_generation_duration: Optional[float] = None,
+        max_generation_duration_beyond_initial: Optional[float] = float("inf"),
         **kwargs,
     ) -> None:
         """Satellite that detects access opportunities for ground locations.
@@ -49,11 +50,16 @@ class AccessSatellite(Satellite):
                 the simulation is infinite.
             initial_generation_duration: [s] Period to calculate opportunities for on
                 environment reset.
+            max_generation_duration_beyond_initial: [s] Maximum time to calculate opportunities
+                beyond the initial generation duration.
             kwargs: Passed through to :class:`Satellite` constructor.
         """
         super().__init__(*args, **kwargs)
         self.generation_duration = generation_duration
         self.initial_generation_duration = initial_generation_duration
+        self.max_generation_duration_beyond_initial = (
+            max_generation_duration_beyond_initial
+        )
         self.access_filter_functions = []
         self.add_access_filter(lambda opportunity: True)
 
@@ -70,6 +76,7 @@ class AccessSatellite(Satellite):
         r_LP_P: np.ndarray,
         min_elev: float,
         type: str,
+        start_time: float = 0.0,
     ) -> None:
         """Add a location to be included in opportunity calculations.
 
@@ -83,10 +90,12 @@ class AccessSatellite(Satellite):
             r_LP_P: [m] Objects planet-fixed location.
             min_elev: [rad] Minimum elevation angle for access.
             type: Category of opportunity target provides.
+            start_time: [s] Time at which to start calculating opportunities for this location.
         """
         location_dict = dict(r_LP_P=r_LP_P, min_elev=min_elev, type=type)
         location_dict[type] = object  # For backwards compatibility, prefer "object" key
         location_dict["object"] = object
+        location_dict["start_time"] = start_time
         self.locations_for_access_checking.append(location_dict)
 
     def reset_post_sim_init(self) -> None:
@@ -111,11 +120,6 @@ class AccessSatellite(Satellite):
         if duration <= 0:
             return
 
-        self.logger.info(
-            "Finding opportunity windows from "
-            f"{self.window_calculation_time:.2f} to "
-            f"{self.window_calculation_time + duration:.2f} seconds"
-        )
         calculation_start = self.window_calculation_time
         calculation_end = self.window_calculation_time + max(
             duration, self.trajectory.dt * 2, self.generation_duration
@@ -124,6 +128,12 @@ class AccessSatellite(Satellite):
             calculation_end / self.generation_duration
         )
 
+        self.logger.info(
+            "Finding opportunity windows from "
+            f"{calculation_start:.2f} to "
+            f"{calculation_end:.2f} seconds"
+        )
+
         # Get discrete times and positions for next trajectory segment
         self.trajectory.extend_to(calculation_end)
         r_BP_P_interp = self.trajectory.r_BP_P
@@ -137,12 +147,18 @@ class AccessSatellite(Satellite):
         r_max = np.max(np.linalg.norm(positions, axis=-1))
         access_dist_thresh_multiplier = 1.1
         for location in self.locations_for_access_checking:
+            start_idx = max(
+                np.searchsorted(times, location["start_time"], side="right") - 1, 0
+            )
+            times_loc = times[start_idx:]
+            positions_loc = positions[start_idx:]
+
             alt_est = r_max - np.linalg.norm(location["r_LP_P"])
             access_dist_threshold = (
                 access_dist_thresh_multiplier * alt_est / np.sin(location["min_elev"])
             )
             candidate_windows = self._find_candidate_windows(
-                location["r_LP_P"], times, positions, access_dist_threshold
+                location["r_LP_P"], times_loc, positions_loc, access_dist_threshold
             )
 
             for candidate_window in candidate_windows:
@@ -153,7 +169,7 @@ class AccessSatellite(Satellite):
                     candidate_window,
                 )
                 new_windows = self._refine_window(
-                    roots, candidate_window, (times[0], times[-1])
+                    roots, candidate_window, (times_loc[0], times_loc[-1])
                 )
                 for new_window in new_windows:
                     self._add_window(
@@ -161,7 +177,7 @@ class AccessSatellite(Satellite):
                         new_window,
                         type=location["type"],
                         r_LP_P=location["r_LP_P"],
-                        merge_time=times[0],
+                        merge_time=times_loc[0],
                     )
 
         self.window_calculation_time = calculation_end
@@ -365,6 +381,7 @@ class AccessSatellite(Satellite):
         self,
         types: Optional[Union[str, list[str]]] = None,
         filter: Union[Optional[Callable], list] = None,
+        min_needed: Optional[int] = None,
     ) -> dict[Any, tuple[float, float]]:
         """Make dictionary of opportunities that maps objects to the next open windows.
 
@@ -372,6 +389,7 @@ class AccessSatellite(Satellite):
             types: Types of opportunities to include. If None, include all types.
             filter: Function that takes an opportunity dictionary and returns a boolean
                 if the opportunity should be included in the output.
+            min_needed: Minimum number of opportunities to return. If None, return all
         """
         if isinstance(types, str):
             types = [types]
@@ -384,11 +402,15 @@ class AccessSatellite(Satellite):
             filter = self.default_access_filter
 
         next_windows = {}
+        total_found = 0
         for opportunity in self.upcoming_opportunities:
             type = opportunity["type"]
             if (types is None or type in types) and filter(opportunity):
                 if opportunity["object"] not in next_windows:
                     next_windows[opportunity["object"]] = opportunity["window"]
+                    total_found += 1
+            if min_needed is not None and total_found >= min_needed:
+                break
         return next_windows
 
     def find_next_opportunities(
@@ -436,8 +458,17 @@ class AccessSatellite(Satellite):
 
                 if len(next_opportunities) >= n:
                     return next_opportunities
+            if (
+                self.window_calculation_time
+                >= self.initial_generation_duration
+                + self.max_generation_duration_beyond_initial
+            ):
+                break
             self.calculate_additional_windows(self.generation_duration)
         if pad and len(next_opportunities) >= 1:
+            self.logger.info(
+                f"Only {len(next_opportunities)} opportunities found, padding to {n}."
+            )
             next_opportunities += [next_opportunities[-1]] * (
                 n - len(next_opportunities)
             )
@@ -457,7 +488,10 @@ class AccessSatellite(Satellite):
         )
 
     def add_access_filter(
-        self, access_filter_fn: Callable, types: Optional[Union[str, list[str]]] = None
+        self,
+        access_filter_fn: Callable,
+        types: Optional[Union[str, list[str]]] = None,
+        prepend: bool = False,
     ):
         """Add an access filter function to the list of access filters.
 
@@ -476,9 +510,14 @@ class AccessSatellite(Satellite):
             def access_filter_type_restricted(opportunity):
                 return opportunity["type"] not in types or access_filter_fn(opportunity)
 
-            self.access_filter_functions.append(access_filter_type_restricted)
+            to_add = access_filter_type_restricted
         else:
-            self.access_filter_functions.append(access_filter_fn)
+            to_add = access_filter_fn
+
+        if prepend:
+            self.access_filter_functions.insert(0, to_add)
+        else:
+            self.access_filter_functions.append(to_add)
 
     @property
     def default_access_filter(self):
@@ -488,12 +527,11 @@ class AccessSatellite(Satellite):
         """
 
         def access_filter(opportunity):
-            return all(
-                [
-                    access_filter_fn(opportunity)
-                    for access_filter_fn in self.access_filter_functions
-                ]
-            )
+            for access_filter_fn in self.access_filter_functions:
+                if not access_filter_fn(opportunity):
+                    return False
+            else:
+                return True
 
         return access_filter
 
@@ -501,6 +539,8 @@ class AccessSatellite(Satellite):
 class ImagingSatellite(AccessSatellite):
     """Satellite with agile imaging capabilities."""
 
+    buffer_name = "image_buffer"
+
     dyn_type = dyn.ImagingDynModel
     fsw_type = fsw.ImagingFSWModel
 
@@ -519,6 +559,7 @@ class ImagingSatellite(AccessSatellite):
         self.dynamics: ImagingSatellite.dyn_type
         self.data_store: "UniqueImageStore"
         self.target_types = "target"
+        self.latest_target = None
 
     @property
     def known_targets(self) -> list["Target"]:
@@ -541,12 +582,7 @@ class ImagingSatellite(AccessSatellite):
         :meta private:
         """
         super().reset_pre_sim_init()
-        self.sat_args["bufferNames"] = [
-            loc["object"].id
-            for loc in self.locations_for_access_checking
-            if hasattr(loc["object"], "id")
-        ]
-
+        self.sat_args["bufferNames"] = [self.buffer_name]
         self.sat_args["transmitterNumBuffers"] = len(self.sat_args["bufferNames"])
 
     def _update_image_event(self, target: "Target") -> None:
@@ -561,16 +597,8 @@ class ImagingSatellite(AccessSatellite):
 
         self._image_event_name = valid_func_name(f"image_{self.name}_{target.id}")
         if self._image_event_name not in self.simulator.eventMap.keys():
-            data_names = np.array(
-                list(
-                    self.dynamics.storageUnit.storageUnitDataOutMsg.read().storedDataName
-                )
-            )
-            data_index = int(np.where(data_names == target.id)[0][0])
             current_data_level = (
-                self.dynamics.storageUnit.storageUnitDataOutMsg.read().storedData[
-                    data_index
-                ]
+                self.dynamics.storageUnit.storageUnitDataOutMsg.read().storedData[0]
             )
 
             def side_effect(sim):
@@ -584,7 +612,7 @@ class ImagingSatellite(AccessSatellite):
                 macros.sec2nano(self.fsw.fsw_rate),
                 True,
                 conditionFunction=lambda sim: self.dynamics.storageUnit.storageUnitDataOutMsg.read().storedData[
-                    data_index
+                    0
                 ]
                 > current_data_level,
                 actionFunction=side_effect,
@@ -638,10 +666,10 @@ class ImagingSatellite(AccessSatellite):
                 behavior.
         """
         self._update_image_event(target)
-        next_window = self.next_opportunities_dict(
-            types=self.target_types,
-            filter=self.default_access_filter,
-        )[target]
+        for opportunity in self.upcoming_opportunities:
+            if opportunity["object"] == target:
+                next_window = opportunity["window"]
+                break
         self.logger.info(
             f"{target} window enabled: {next_window[0]:.1f} to {next_window[1]:.1f}"
         )
@@ -671,9 +699,10 @@ class ImagingSatellite(AccessSatellite):
         """
         msg = f"{target} tasked for imaging"
         self.logger.info(msg)
-        self.fsw.action_image(target.r_LP_P, target.id)
+        self.fsw.action_image(target.r_LP_P, self.buffer_name)
         self.enable_target_window(target, max_duration=max_duration)
         self.draw_imaging_line(target)
+        self.latest_target = target
 
     @vizard.visualize
     def draw_imaging_line(
